﻿
using System;

namespace WindowsFormsApplication1
{
    public delegate double function(double value);

    class TLayer
    {
        public int featuresIn;
        public int featuresOut;
        public double[] x;
        public double[] o;
        public double[,] w;
        public double[] offs;
        public double[] deltaOffs;
        public double[] gradient;
        public double[,] gradientMatrix;
        public double[,] deltaW;
        public bool calcSum = true;
        public double softMaxSum;
        public bool isMatrixGradient = false;

        public function Act;
        public function dAct;

        public TLayer(int iFeaturesIn, int iFeaturesOut)
        {
            int j, k;
            featuresIn = iFeaturesIn;
            featuresOut = iFeaturesOut;
            w = new double[featuresIn, featuresOut];
            o = new double[featuresOut];
            offs = new double[featuresOut];
            deltaOffs = new double[featuresOut];
            deltaW = new double[featuresIn, featuresOut];
            x = new double[featuresOut];  
            
            for (j = 0; j < featuresOut; j++)
            {
                offs[j] = 0;
                deltaOffs[j] = 0;
                for (k = 0; k < featuresIn; k++)
                {
                    w[k, j] = 0.5;
                    deltaW[k, j] = 0;
                }
            }
            Act = SoftMax;
            dAct = dSoftMax;
        }

        public void InitSigmoid()
        {
            gradient = new double[featuresOut];
            Act = Sigmoid;
            dAct = dSigmoid;
        }

        public void InitTanH()
        {
            gradient = new double[featuresOut];
            Act = TanH;
            dAct = dTanH;
        }

        public void InitSoftMax()
        {
            gradientMatrix = new double[featuresIn, featuresOut];
            Act = SoftMax;
            dAct = dSoftMax;
            isMatrixGradient = true;
        }

        public double Sigmoid(double x)
        {
            return 1.0 / (1.0 + Math.Exp(- x));
        }

        public double dSigmoid(double x)
        {
            return  x * (1 - x);
        }

        public double TanH(double x)
        {
            return ((Math.Exp(2.0 * x) - 1.0) / (Math.Exp(2.0 * x) + 1.0) / 2.0) + 0.5;
        }

        public double dTanH(double x)
        {
            return (1 - x * x) / 2;
        }

        public double SoftMax(double xIn)
        {
            int i;
            if(calcSum)
            {
                softMaxSum = 0;
                for(i=0; i < featuresOut; i++)
                {
                    softMaxSum += Math.Exp(x[i]);
                }
                calcSum = false;
            }  
            return -Math.Log(Math.Exp(xIn)/softMaxSum);
        }

        public double dSoftMax(double xIn)
        {
            int i;
            if (calcSum)
            {
                softMaxSum = 0;
                for (i = 0; i < featuresOut; i++)
                {
                    softMaxSum += Math.Exp(x[i]);
                }
                calcSum = false;
            }
            return Math.Exp(xIn) / softMaxSum;
        }

        public double Neutral(double x)
        {
            return x;
        }

        public double dNeutral(double x)
        {
            return 1;
        }
    }
}
